/*
 * File:
 *
 * Contains:
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2013 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __ETHERMESH_H
#define __ETHERMESH_H
#include "Koliada.h"
#include "timer.h"

#include "radio.h"
#include "seqCounter.h"

#define kNodeNameSize		32
#define kNeighborTableSize 8

typedef UInt16 emAddr;
typedef byte emStatus;
#define BCAST_ADDR ((emAddr)0xffff)

typedef void (*SyncLostCallback)(byte state);
typedef void (*NeighborUpdateCallback)(emAddr addr, signed char rssi);

enum
	{
	emFrameDropped = 1,
	emNodeMeshed,
	emNodeRenamed,
	emResolvedName,
	emResolvedAddr,
	emResolveNameReq,
	emResolveAddrReq,
	emRenameNodeReq,
	};

void initEtherMESH(word syncInterval, byte syncWindow, byte syncTimout, byte msgPendCount);
// Meshing is NOT started on init - call emMeshStart() to begin meshing

void emMeshStart();				// (re)start meshing (must be inited)
void emMeshStop();				// stop meshing

enum emCntlOpts
	{
	emNodeNameOpt = 1,			// get/set node name
	emUseListenWindowOpt,
	emNeighborUpdateCallback,	// get/set neighbor table update callback - posted whenever the neighbor table changes
	emSyncLostCallback,			// get/set sync lost callback - posted whenever the sync state changes (0 == lost)
	emMeshPower,					// get/set mesh Tx power
	emMeshChannel,					// get/set mesh channel
	};
STATUS emSetCntl(byte opt, ...);

EVENT emStatMsg(EVENT);			// EtherWIRE status event
EVENT emRecvMsg(EVENT);			// message receive event
bool  emSendMsg(emAddr dst, byte *buf, byte len);

void emStopResolver();			// stop the name resolver (stopped on mesh stop)
void emStartResolver();			// start the name resolver (started on mesh start)
void emFlushResolverCache();	// flush the name resolver cache

emAddr emResolveAddr(char *);	// resolve node name to addr (via cache)
char  *emResolveName(emAddr);	// resolve addr to node name (via cache)
void   emRenameNode(emAddr, char *);	// rename node @ addr to name

void emAsyncResolveAddr(char *, HANDLER, word timeout);	// resolve node name to addr
void emAsyncResolveName(emAddr, HANDLER, word timeout);	// resolve addr to node name

void showEtherMesh(char const *func, word line);	// show (debug) internal state

extern emAddr nodeAddr;			// the nodes EtherMESH addr (read only!)
extern char nodeName[];
byte randomByte();

byte getNeighborCount();		// # currently active neighbors
void getNeighborEntry(byte index, void **result);
void emFlushNeighborTable();	// flush the neighbor cache

#endif//__ETHERMESH_H
