/*
 *	File:	Koliada - fsm.h
 *
 *	Contains: Koliada FSM header
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright:	 1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __FSM_H
#define __FSM_H

#define STRUCT const struct

struct State;			// forward decl
struct StateEvent;	// forward decl
struct StateMachine;	// forward decl

typedef const char *smEvent;	// state event (defined by app)
typedef const char *smState;	// state ident (defined by app)
typedef void *StateData;		// state data (app specific)

typedef void (*smAction)(STRUCT State *this, STRUCT StateEvent *);

typedef STRUCT State
	{
	smState state;						// state enum for this state
	StateData data;					// state data 'segment'
	smAction arrive;					// state arrival function
	smAction depart;					// state departure function
	unsigned char eventCount;		// number of events defined
	STRUCT StateEvent *events;		// ways out of this state
	} State;

typedef void (*smTransform)(State *old, State *new);

typedef STRUCT StateEvent
	{
	smEvent E;						// event that drives this transition
	smTransform T;					// event transition function
	State *newState;				// state after the transition
	} StateEvent;

typedef void (*smTransit)(struct StateMachine *, smEvent);

typedef struct StateMachine
	{
	State *thisState;			// current state
	State *errorState;		// error state

#if 0
#warning
	struct
		{
		word debug :1;			// show debug info
		};
#endif

	smTransit wait;			// called when about to transition
	smTransit post;			// called when finshed transition

#if 1
	unsigned char stateCount;	// number of states defined
	State **stateTable;			// table of valid states
#endif
	} StateMachine;

#define DEF_EVENT(NAME) const char NAME[] = #NAME
#define DEF_STATE(NAME) const char NAME[] = #NAME ; static State NAME##State

#define EXT_EVENT(NAME) extern const char NAME[]
#define EXT_STATE(NAME) extern const char NAME[]

void smInit(StateMachine *sm);
void smStart(StateMachine *sm, smEvent e);	// start with initial event e

// execute a state event transition
// pass event data as vargs into _smTransit!
#if 1
	smState _smTransit(const char *, unsigned, StateMachine *sm, smEvent e);
	#define smTransition(sm, e) _smTransit(__FILE__, __LINE__, sm, e)
#else
	smState _smTransit(StateMachine *sm, smEvent e);
	#define smTransition(sm, e) _smTransit(sm, e)
#endif

EXT_EVENT(eError); // always defined
EXT_STATE(Error);  // always defined

#endif//__FSM_H

