/*
 *	File: sysPower.h
 *
 *	Contains: system power management API
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright:	 1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __SYSPOWER_H
#define __SYSPOWER_H

// how these are scaled to the hardware is largely system dependent
typedef enum
	{
	kSysWake = 0,		// full on (moderated by SysSystemSpeed if available)
	kSysSnooze = 1,	//
	kSysSleep = 2,		//
	kSysDown = 3,		// lowest power mode

	kSysIdle = 256,	// on but cpu is powered down
	} SysPowerMode;

typedef enum
	{
	kSysFastest,
	kSysFast,
	kSysSlow,
	kSysSlowest,
	} SysSystemSpeed;

SysPowerMode   sysSetPowerMode(SysPowerMode);		// set a given power mode
SysPowerMode   sysGetPowerMode();						// get the current power mode
SysPowerMode   sysSetSleepMode(SysPowerMode);		// set a given sleeping power mode
SysPowerMode   sysGetSleepMode();						// get the current sleeping power mode
SysSystemSpeed sysSetSystemSpeed(SysSystemSpeed);	// set the system speed we want
SysSystemSpeed sysGetSystemSpeed();						// get the current system speed

void sysSleep(SysPowerMode mode);

SysPowerMode (*getEventSleepNotice())(SysPowerMode);
SysPowerMode (*setEventSleepNotice(SysPowerMode (*notify)(SysPowerMode)))(SysPowerMode);

#endif//__SYSPOWER_H
