/*
 *	File: Koliada - part.h
 *
 *	Contains:
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2010 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __PART_H
#define __PART_H

// map to the apporiate part support headers

#define NOP() asm("NOP")

// parameters for sysSetMainClock
#define CLOCK_SRC_XOSC      0  // High speed Crystal Oscillator Control
#define CLOCK_SRC_HFRC      1  // Low power RC Oscillator

// parameters for sysSelect32kClock
#define CLOCK_32K_XTAL      0  // 32.768 Hz crystal oscillator
#define CLOCK_32K_RCOSC     1  // 32 kHz RC oscillator

void sysSetMainClock(byte source);
byte sysSelect32kClock(byte source);

void setPowerMode(byte);
word mask2bit(word mask);

// enable one of these for access to architecture specific SFRs, etc
// these are (generally) >optional< at the target level since they reduce portability!
#if CC1110 || chip == 1110
	#if CCTARGET
		#error CCTARGET is has already been defined!!
	#endif
	#define CCTARGET 1
	#include "ioCCxx10_bitdef.h"
	#include "ioCC1110.h"		// SoC specific header
#endif
#if CC2530
	#if CCTARGET
		#error CCTARGET is has already been defined!!
	#endif
	#define CCTARGET 1
	#include "bitdefs.h"
	#include "ioCC2530.h"		// SoC specific header
#endif
#if CC2531
	#if CCTARGET
		#error CCTARGET is has already been defined!!
	#endif
	#define CCTARGET 1
	#include "bitdefs.h"
	#include "ioCC2531.h"		// SoC specific header
#endif
#if CC2540
	#if CCTARGET
		#error CCTARGET is has already been defined!!
	#endif
	#define CCTARGET 1
	#include "bitdefs.h"
	#include "ioCC2540.h"		// SoC specific header
#endif
#if CC2541
	#if CCTARGET
		#error CCTARGET is has already been defined!!
	#endif
	#define CCTARGET 1
	#include "bitdefs.h"
	#include "ioCC2541.h"		// SoC specific header
#endif
#if CC25XX
	#if CCTARGET
		#error CCTARGET is has already been defined!!
	#endif
	#define CCTARGET 1
	#include "ioCC2530.h"		// SoC generic header
#endif
#ifndef CCTARGET
	// if you don't use specific SoC include, then must define CCTARGET externally!
	#error CCTARGET is not defined
#endif

#if defined(CC1110)
	// Bauderate mantissa and exponent
	// Baudrate = (((256 + BAUD_M) * 2^BAUD_E) * F) / 2^28
	#if 0
		// Baudrate = 57.6 kbps (U0BAUD.BAUD_M = 34, U0GCR.BAUD_E = 11)
		#define UART_BAUD_M  34
		#define UART_BAUD_E  11
	#endif
	#if 1
		// Baudrate = 115200 kbps (U0BAUD.BAUD_M = 34, U0GCR.BAUD_E = 12)
		#define UART_BAUD_M  34
		#define UART_BAUD_E  12
	#endif

	#define kSystemDataSegmentSize 0x100	// size of the system data segment
#endif

#if (defined(CC2530) || defined(CC2531) || defined(CC2540) || defined(CC2541))
#if 0
	// Baudrate =  115200 bps (BAUD_M = 216, BAUD_E = 11)
	// Baudrate =  230400 bps (BAUD_M = 216, BAUD_E = 12)
	// Baudrate =  230400 bps (BAUD_M = 216, BAUD_E = 13)
	// Baudrate =  921600 bps (BAUD_M = 216, BAUD_E = 14)
	// Baudrate = 1843200 bps (BAUD_M = 216, BAUD_E = 15)
	// Baudrate = 1996094 bps (BAUD_M = 255, BAUD_E = 15)
	// Baudrate = 3992188 bps (BAUD_M = 255, BAUD_E = 16)
	#define UART_BAUD_M  255
	#define UART_BAUD_E  15
#endif

	#define kSystemDataSegmentSize 0x200	// size of the system data segment

	// ADC
	// single ended
	#define AIN0	0x0
	#define AIN1	0x1
	#define AIN2	0x2
	#define AIN3	0x3
	#define AIN4	0x4
	#define AIN5	0x5
	#define AIN6	0x6
	#define AIN7	0x7

	// differential
	#define AIN01	0x8
	#define AIN23	0x9
	#define AIN45	0xa
	#define AIN67	0xb
	#define AGND	0xc
	#define ATEMP	0xe
	#define AVDD	0xf

	// decimation
	#define DEC64 0				//
	#define DEC128 BIT4			//
	#define DEC256 BIT5			//
	#define DEC512 (BIT5|BIT4)	//

	// voltage reference
	#define IREF 0					// internal
	#define XREF BIT6				// external on AIN7
	#define VREF BIT7				// AVDD5 pin
	#define XREF67 (BIT7|BIT6)	// External on AIN6/7

	int adcTemp();				// adc temp xx.xx *C
	word adc(byte spec);			// adc.c
#endif

#endif//__PART_H
