// support for the PTH sensor

#include "EtherDATA.h"
#include "Tag.ddl.h"
#include "string.h"

#include "MS8607.h"

#define kPTHSampleInterval 32	// seconds

double calcHumRel(UInt16 rawH);
double calcHumTmp(UInt16 rawT);
double *calcTempPress();

int humiState = 0;
UInt16 TempHumid[3];

// Length of sensor data in bytes
#define HUMIDITY_DATA_LEN               4
#define kTempType 	      kFirstpCacheType + 1		// cached record type
#define kHumiType	         kFirstpCacheType + 2		// cached record type

// read PTH from MS8607 sensor
int *samplePTH()
	{
	UInt8 hData[HUMIDITY_DATA_LEN];
	unsigned long pData;
	unsigned long tData;
	UInt16 humid_MSB;
	UInt16 humid_LSB;
	UInt16 rawHumid;
	static int data[3];
	double P, H, T;

#if 0
	SensorInit();
	sysDelayMs(20);

	while (humiState < 2)
		{
		HumiExecMeasurementStep(humiState);
		humiState++;
		}

	if (humiState == 2)
		{
		if (HumiReadMeasurement(hData))
			{
			humid_LSB = hData[2];
			humid_MSB = hData[3] << 8;
			rawHumid = (humid_LSB | humid_MSB);
			H = calcHumRel(rawHumid);
			}
		humiState = 0;
		}

	PressureMeasurementStep();
	PresReadMeasurement(&pData);

	TempMeasurementStep();
	TempReadMeasurement(&tData);

	double *values = calcTempPress();
	P = values[0];
	T = values[1];

#if DEBUG
	debug("P=%d.%d\n", (int) P, (int)((P - (int) P) * 1000));
	debug("H=%d.%d\n", (int) H, (int)((H - (int) H) * 1000));
	debug("T=%d.%d\n", (int) T, (int)((T - (int) T) * 1000));
#endif

	data[0] = (int) P;
	data[1] = (int) H;
	data[2] = (int) T;
#if DEBUG
#warning PHT truncated by integer conversion!!!
#endif
#else
	data[0] = 1111;
	data[1] = 2222;
	data[2] = 3333;

   dbSet(humiValue, data);
#endif
	
	return data;
	}

static UInt16 sampleCount;
static UInt16 sampleRemain;
static UInt16 samplePeriod;

// define PTH sample timer
static DefineTimer(sampleTimer);
static void sampleTimerHandler(TIMER t)
	{
	if (sampleCount)
		{
		sampleCount--;
		cmSetTimer(sampleTimer, TICKS(30000));
		return;
		}
	sampleCount =  samplePeriod / 30;
	sampleRemain = samplePeriod % 30;

	// start sampling
	cmSetTimer(sampleTimer, TICKS(sampleRemain * 1000));
	// this will drift slowly

	debug(".");

	int *p = samplePTH();

#if 0
	string desc;
	string unit;

	//UInt8 hData[HUMIDITY_DATA_LEN];
	//UInt16 temp_MSB;
	//UInt16 temp_LSB;
	//UInt16 rawTemp;
	//UInt16 humid_MSB;
	//UInt16 humid_LSB;
	//UInt16 rawHumid;

		while(humiState<3)
				{
					HalHumiExecMeasurementStep(humiState);
					humiState++;
					sysDelayMs(7);
				}

		if (humiState == 3)
		  {
		        //readHumData();
		       if (HalHumiReadMeasurement(hData))
		          {
		            temp_LSB = hData[0];
		            temp_MSB = hData[1]<<8;
		            rawTemp = (temp_LSB | temp_MSB);
		            humid_LSB = hData[2];
		            humid_MSB = hData[3]<<8;
		            rawHumid = (humid_LSB | humid_MSB);
		            TempHumid[0] = calcHumTmp(rawTemp);
		            TempHumid[1] = calcHumRel(rawHumid);
		          }
		        humiState = 0;
		        dbSet(humiValue, TempHumid);
		        humi_timer_counter = 0;
		        dbGetAttribute(ref, humiValueEnum.units, &unit);
		        dbGetAttribute(ref, humiValueEnum.description, &desc);
		        printf("%s: %u, %u\n", desc, TempHumid[0], TempHumid[1]);
		   }
		*/
		memcpy(TempHumid, sampleHumiTemp(), sizeof(TempHumid));
		memset(&TempHumid[2], 0, 1);
		dbSet(humiValue, TempHumid);
		humi_timer_counter = 0;
		dbGetAttribute(ref, humiValueEnum.units, &unit);
		dbGetAttribute(ref, humiValueEnum.description, &desc);
		printf("%s: %u, %u\n", desc, TempHumid[0], TempHumid[1]);
		}
#endif
	}

void startPTH(word interval)
	{
	// start sampling with interval second period
	samplePeriod = interval;
	sampleCount = sampleRemain = 0;

	// do an immediate sample (also starts sampling)
	sampleTimerHandler(sampleTimer);
	}

void initPTH()
	{
	//
	// The MS8607 is an integrated pressure, temp & humidity sensor
	// We set up the sensor to measure all three values at once

	// set up and start the humidity sampling timer
	objectCreate(sampleTimer, kIntervalTimer, 0);
	OnEvent(sampleTimer, (HANDLER) sampleTimerHandler);

	// publish the data
	dbPublish(humiValue);

	// start sampling
	startPTH(kPTHSampleInterval);
#if 0
	while (1)
		{
		int *p = samplePTH();
		cmDelayMs(1000);
		}
#endif
	}

