#include "Koliada.h"
#include "EtherDATA.h"
#include "radio.h"
#include "pCache.h"
#include "sysGpio.h"
#include "sysPower.h"
#include "string.h"
#include "dbData.ddl.h"
#include "led.h"

#define LED_RED 0	// virtual LED pin
#define LED_YEL 1
#define LED_GRN 2

#define TEST_TRACKER 0
#define TEST_STATION 0
#define MAX_PING_MISSED 5

int printf(const char *, ...);
void initUI();
void initButton();
void initLED();
void rssiGetHandler(string remote);
void rssiGet(dbValue ref, string group);
void initRSSI();
void ledOn(word led, word delay);
void init_recovery();

int sCompare(void *key, void *buf, SIZE_T size);
void initSubscriptions(void);

////////////////////////////////////////////////////////////////////////////////
#define PCACHE_ADDR ((ROMPTR) 0xB000)
#define NUM_PAGES 4
#define PAGE_SIZE 2048
#define NUM_RESERVE 1

#define kSubscribeType        kFirstpCacheType		// cached record type
#define kConfigType           kFirstpCacheType + 1
////////////////////////////////////////////////////////////////////////////////
void *cache = (void *) PCACHE_ADDR;

boolean checkFlush(wyde type, void *buf, SIZE_T size)
	{
	// called when a block is about to be flushed
	switch (type)
		{
		case kSubscribeType:
			return 1;           // yes, we can flush these... for now

		case kConfigType:
			return 1;
		}
	return 0;	// keep everything else
	}
//////////////////////////////////////////////////////////////////////////////////

/* *****************************************************************************
Called when configValue is changed on the device;
For the station, the device given the new name.
Update: new Request value is not being used, solely the new name
********************************************************************************/
STATUS configRequest(OBJECT o)
	{
	dbValue configValue = (dbValue)o;
	string data;
	char newData[5];
	byte request = 0;

	memset(newData, 0, sizeof(newData));

	dbGet(configValue, &data);
	strnbuf(5, newData, 0, "%s", data);

	memcpy(&request, &newData[4], sizeof(request));
	memset(&newData[4], 0, 1);

	strcpy(data, newData);


	if (strcmp("0000", newData))
		{
		//printf("new name: %s\n", data);
		dbRename(nodeAddr, data);
		}
	if (request >= '0')
		{
		request = request - '0';
		}

	delChunk(kConfigType, NULL, 0);       // delete old threshold value from pCache
	newChunk(kConfigType, &request, 1);            // save new threshold value in pCache

	if (request == 2) // if configured as a Gateway, stop publishing to deviceType
		{
		dbWithdraw(deviceType);
		}


	ledOn(LED_GRN, 200);

	return 0;
	}


#if TEST_TRACKER
void deviceTypeHandler()
	{
	ledOn(LED_RED, 100);
	printf("DTR\n");
	}
#endif

char *sysGetEnv(const char *name);
int sysSetEnv(const char *name, const char *value, int overwrite);

int main(int argc, char **argv)
	{
	UInt8 *temp;
	byte request;
	UInt16 interval = 10000;
	setMeshPendCount(1);
	setMeshWindow(128);
	setMeshInterval(interval);

	//printf("Mesh Interval is set at %d ms \n", interval);

	// check we've been given a name as part of the config
	// null name simply uses the node address as the name
	dbInit(sysGetEnv("emNodeName"));

	emSetCntl(emMeshChannel, 15);

	emMeshStart();

	printf("Node %0x started as %s \"%s\"\n\n", nodeAddr, argv[0], nodeName);

	GPIO.XOR(LED_RED);
	cmDelayMs(100);
	GPIO.XOR(LED_RED);
	//void initSerialBios(); initSerialBios();

	//pCacheInit(cache, NUM_PAGES, PAGE_SIZE, NUM_RESERVE, checkFlush, true);
	pCacheInit(cache, NUM_PAGES, PAGE_SIZE, NUM_RESERVE, checkFlush, false);
	initUI();
	initButton();
	initLED();
	//initRSSI();
	initSubscriptions();

	if ((temp = locChunk(kConfigType, NULL, 0)) == NULL);
	else
		{
		memcpy(&request, temp, 1);
		if (request == 2) // if configured as a Gateway, stop publishing to deviceType
			{
			dbWithdraw(deviceType);
			}
		}

	//init_recovery();
	
#if TEST_STATION
	cmStartTimer(SampleTimer, deviceType, 1);
	//dbSubscribe("*", rssiTable, (HANDLER)rssiGet);
	dbWithdraw(rssiTable);
#endif

#if TEST_TRACKER
	dbSubscribe("*", deviceType, (HANDLER)deviceTypeHandler);
	dbWithdraw(deviceType);
#endif
	ledOn(LED_RED, 100);
	// be awake to service uarts
	setEventSleepMode(kSysSnooze);

	// wait for events
	WaitEvent(0, 0);// never returns
	}
