#include "Koliada.h"
#include "EtherDATA.h"
#include "radio.h"
#include "pCache.h"
#include "sysGpio.h"
#include "sysPower.h"
#include "string.h"
#include "dbData.ddl.h"
#include "led.h"

#define MAX_PING_MISSED 5

int printf(const char *, ...);
byte gatewayTimer_counter = 0;
byte gatewayPingTimer_counter = 0;
byte ping_counter = 0;

/*******************************************************************************
RECOVERY LOGIC!
- Gateway devices send out a value under dataObject 'gateway' every 'x' minutes
- Peripherals (trackers, stations, badges, etc). are subscribed to 'gateway'
- Everytime a peripheral receives that 'gateway' ping, it resets its counter
- IF peripheral does not receive a 'ping' in over 'MAX_PING_MISSED' minutes, it resets its network

PURPOSE:
-To make sure that every device is 'grouped' with a gateway device
*******************************************************************************/

// gateway ping timer, sends out a 'ping' every x minutes
DefineTimer(gatewayPingTimer);
void gatewayPingTimerHandler(TIMER e, dbValue ref, UInt16 time)
	{
	gatewayPingTimer_counter ++;

	if (gatewayPingTimer_counter >= 2 * time)
		{
		gatewayPingTimer_counter = 0;
		dbSet(gateway, 1);
		printf("gateway ping! \n");
		}
	}

void gatewayHandler(string group, dbValue ref, void *value) // CALLED everytime gateway ping is received
	{
	ping_counter = 0;
	//MemledOn(LED_GRN, 50);
	printf("ping received:%s\n", group);
	}

DefineTimer(gatewayDelayTimer);
void gatewayDelayTimerHandler(TIMER e)
	{
	emMeshStart();
	}

DefineTimer(gatewayTimer);
void gatewayTimerHandler(TIMER e)
	{
	gatewayTimer_counter++;
	if (gatewayTimer_counter >= 2) //every minute
		{
		gatewayTimer_counter = 0;
		ping_counter ++; // add ping counter
		if (ping_counter >= MAX_PING_MISSED) // if ping_counter is greater than threshold, reset network
			{
			printf("missed pings\n");
			ping_counter = 0;
			emMeshStop();
			cmStartTimer(gatewayDelayTimer);
			}
		}
	}

void init_recovery() // creates and starts timer, subscribes to gateway 'ping'
	{
	// set up gateway Ping timer
	objectCreate(gatewayPingTimer, kIntervalTimer, TICKS(30000));
	OnEvent(gatewayPingTimer, (HANDLER) gatewayPingTimerHandler);

	dbSubscribe("*", gateway, (HANDLER)gatewayHandler);

	// set up and start the gateway Ping timer
	objectCreate(gatewayTimer, kIntervalTimer, TICKS(30000));
	OnEvent(gatewayTimer, (HANDLER) gatewayTimerHandler);
	cmStartTimer(gatewayTimer);

	// set up and start the gateway Ping timer
	objectCreate(gatewayDelayTimer, kSingleShotTimer, TICKS(128));
	OnEvent(gatewayDelayTimer, (HANDLER) gatewayDelayTimerHandler);
	}