#include <string.h>
#include "EtherDATA.h"
#include "dbData.ddl.h"

// here we are looking at >remote< rssi data which is mapped to emNeighborTable

#define krssiSampleInterval 	TICKS(30000)
#include "BoardConfig.h"
//#include "interrupt.h"
#include "system.h"

#define UxCSR(u) (*((volatile byte *)0x7000 + u->csr))
#define UxUCR(u) (*((volatile byte *)0x7000 + u->ucr))
#define UxGCR(u) (*((volatile byte *)0x7000 + u->gcr))
#define UxBUF(u) (*((volatile byte *)0x7000 + u->buf))
#define UxBAUD(u) (*((volatile byte *)0x7000 + u->baud))

#define RX_READY      0x04
#define UART_BUSY     0x01

static UartDescriptor *uart;	// system uart

int output(const char *fmt, ...);
void rssiGetHandler(string remote);
void timerX(TIMER t, dbValue ref, UInt16 time);
UInt16 timer_counter;
string devicename;

typedef struct
	{
	word addr ;
	signed char rssi ;
	} NeighborTable;

NeighborTable table[8];


static STATUS rssiHandler(string group, dbValue ref, void *cookie, byte index, void *value)
	{
	NeighborEntry item = (NeighborEntry) value;

	if (item->addr == 0)
		return kScanStop;
	string name = emResolveName(item->addr);

	if (index)
		output(",");

	if (name)
		output("{\"name\":\"%s\",\"addr\":%x,\"rssi\":%d}", name, item->addr, item->rssi);
	else
		output("{\"name\":null,\"addr\":%x,\"rssi\":%d}", item->addr, item->rssi);

	return kScanContinue;
	// return value has actions
	// 0 - continue scan
	// 1 - stop scan
	}

void rssiGetHandler(string remote)
	{
	extern string owner;
	dbGet(emNeighborTable, NULL);
	//dbGetValue(remote, emNeighborTable, NULL, 30);// force a get!!
	UxCSR(uart) &= ~(0x40);
	output("{\"emNeighborTable\":\"%s\",\"set\":[", remote);
	dbScanArrayFrom(remote, emNeighborTable, NULL, rssiHandler, 10);
	output("]}\n");
	UxCSR(uart) |= (0x40);
	}

void rssiGet(dbValue ref, string group)
	{
	output("{\"emNeighborTable\":\"%s\",\"set\":[", group);
	dbScanArray(ref, NULL, rssiHandler);
	output("]}\n");
	}

// define rssi sample timer
DefineTimer(rssiSampleTimer);
void rssiSampleTimerHandler(TIMER e, dbValue ref, UInt16 time)
	{
	timer_counter++;
	if (timer_counter >= time * 2)
		{
		timer_counter = 0;
		dbGet(emNeighborTable, &table);
		dbSetValue(NULL, rssiTable, &table);
		output("{\"emNeighborTable\":\"%s\",\"set\":[", nodeName);
		dbScanArray(ref, NULL, rssiHandler);
		output("]}\n");
		}
	}

void initRSSI()
	{
	// set up and start the accelerometer sampling timer
	objectCreate(rssiSampleTimer, kIntervalTimer, TICKS(krssiSampleInterval));
	OnEvent(rssiSampleTimer, (HANDLER) rssiSampleTimerHandler);

	cmStartTimer(rssiSampleTimer, rssiTable, 1);

	dbPublish(rssiTable);
	}

