/*
 * File:
 *
 * Contains:
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2014 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#include <stdio.h>
#include <string.h>

#include "Koliada.h"
#include "target.h"
#include "timer.h"
#include "sysGpio.h"
#include "dbData.ddl.h"
#include "pCache.h"
#include "EtherDATA.h"

#define kConfigType           kFirstpCacheType + 1

int output(const char *fmt, ...);
void uartInit(byte port, void (*rxHandler)());
byte uartGets(byte *buf, byte len);
void rssiGetHandler(string remote);
void rssiGet(dbValue ref, string group);
ExternTimer(gatewayTimer);
ExternTimer(gatewayPingTimer);

/*******************************************************************************
UI.c contains the logic for receivng serial data, JSON parsing and button pushes
********************************************************************************/
char *expect(char *input, char *str, byte len)
	{
	if (strncmp(input, str, len) != 0)
		{
		debug("%s\n", input);
		if (!strcmp(input, "[mcusb]\n")) // looking for [mcusb], if it is, device must be a gateway
			{
			dbUnsubscribe("*", gateway); // -> unsubscribe to gateway
			dbPublish(gateway);// start publishing gateway dataObject
			cmStopTimer(gatewayTimer);
			cmStopTimer(gatewayPingTimer);
			cmStartTimer(gatewayPingTimer, gateway, 1); // start gateway ping timer
			debug("gatewayTimer started \n");
			}
		debug("^ -- expected '%s'\n", str);
		return NULL;
		}
	return input += len;
	}

char *contains(char *input, char *set, char *expected)
	{
	char *s;
	for (; *set; set++)
		if ((s = strchr(input, *set)))
			return s;

	debug("%s\n", input);

	debug("^ -- expected '%s'\n", expected);
	return NULL;
	}

#if 0
// parser tests
char tests[] =
	{
	{"cmd": "test"
	, "args":
		["A"]}
	{"cmd": "test"
	, "args":
		["A b", "bodyTemp"]}
	{"cmd": "test"
	, "args":
		[1, "bodyTemp"]}
	{"cmd": "test"
	, "args":
		[true, "bodyTemp"]}
	{"cmd": "test"
	, "args":
		[false, "bodyTemp"]}
	{"cmd": "test"
	, "args":
		[1, null]}
	{"cmd": "test"
	, "args":
		[]}
	};
#endif
void exec(char *fn, byte argc, char **argv, va_list args);

DefineEvent(jsonEvent);
void jsonEventHandler(EVENT e, char *cmd, word len)
	{
	// only supported input is a JSON string of the form;
	//		{"cmd":"function","args":["arg0","arg1",...]}
	//	where;
	//		function: the name of any supported function
	//		arg0...N: any of JSON INTEGER, NULL, TRUE, FALSE or STRING
	//		NO WHITE SPACE (except arg strings as appropriate)
	//
//	debug("%s: %s", __func__, cmd);

	// here we parse the cmd
	char next, *s, *p = (char *)cmd;
	char *fn;
	byte argc;
	char *argv[8];
	void *args[8];
	bool img_FLAG = FALSE;
//	bool img_qt_FLAG = FALSE;
//	UInt8 img_22_cnt = 26;
	if (!(p = expect(p, "{\"cmd\":\"", 8)))
		return;

	fn = p;
	if (!(s = contains(p, "\"", "<function>")))
		return;
	*(p = s) = 0;
	p++;

	if (!(p = expect(p, ",\"args\":[", 9)))
		return;

	for (argc = 0, next = *(s = p); next != ']'; next = *s, *(p = s) = 0, p++)
		switch (*p)
			{
			case ' ':
			case '\t':
			case ',':
				s = p++;
				break;

			case '\"':	// string
				p++;
				if (img_FLAG)
					{
					argv[argc] = p;
					args[argc] = (void *) p;
					argc++;
					// s += 23;
					p += 24;
					goto done;
					}
				if (!(s = contains(p, "\"", "\"<string>\"")))
					return;
				argv[argc] = p;
				args[argc] = (void *) p;
				argc++;
				*s = 0;
				s++;	// delete trailing \"
				if (!strcmp(p, "imgData"))
					{
					img_FLAG = TRUE;
					break;
					}
				break;

			case '0':	// digit
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				if (!(s = contains(p, ",]", "<number>,")))
					return;
				argv[argc] = p;
				args[argc] = (void *) atoi(p);
				argc++;
				break;

			case 't':	// true
				if (!(s = expect(p, "true", 4)))
					return;
				argv[argc] = p;
				args[argc] = (void *) true;
				argc++;
				break;

			case 'f':	// false
				if (!(s = expect(p, "false", 5)))
					return;
				argv[argc] = p;
				args[argc] = (void *) false;
				argc++;
				break;

			case 'n':	// null
				if (!(s = expect(p, "null", 4)))
					return;
				argv[argc] = p;
				args[argc] = 0;
				argc++;
				break;
			}

	if (*p == ']')
		{
		// no args
		if (!(p = expect(p, "]}", 2)))
			return;
		}
	else
		{
		if (!(p = expect(p, "}", 1)))
			return;
		}
done:
	// now do something useful!
	exec(fn, argc, argv, args);
	}

void serialRxReady()
	{
	// recover the incoming buffer
	// running on interrupt so we can't call serialPuts!!
	// we post an event to let the interrupt complete
	// then we handle the event in 'user mode'
	static byte rxBuf[256];
	PostEvent(jsonEvent, rxBuf, uartGets(rxBuf, sizeof(rxBuf) - 1));
	debug(">> %s", rxBuf);
	}

static void __assertFailed(char *name, unsigned line, char *exp)
	{
	char *rindex(const char * path, int c);
	void abort(void);
	static bool inAssert = false;
	char *s = rindex(name, '\\');
	if (s) name = ++s;

	debug("\n>>>> ");
	if (inAssert)
		{
		debug("\n%s.%u: assert(%s) - failed!!\n\n", name, line, exp);
		return;
		}
	inAssert = true;
	debug("\n\n");

	rfRadioOff();
	debug("listen=disabled\n");
//	cmStopTimer(syncTimer); debug("sync=disabled\n");

	// show some stuff
	showEtherMesh(__FUNC__, __LINE__);

	// show the problem
	debug("%s.%u: assert(%s) - failed!!\n", name, line, exp);

	// handle key events only
	do
		{
		WaitEvent(ANY, 0);
		}
	while (sysGetch() != 0x1b);

	// we can restart...
	rfEnableRx();
	debug("listen=enabled\n");
	inAssert = false;
	}

void buttonPressed(int button)
	{
	UInt8 *temp;
	byte request = 0;

	printf("MESH button pressed\n");
	if ((temp = locChunk(kConfigType, NULL, 0)) == NULL)
		{
		dbSet(deviceType, 1);
		}
	else
		{
		memcpy(&request, temp, 1);
		}

	if (request == 1)
		{
		dbSet(deviceType, 1);
		}
	string name = emResolveName(nodeAddr);
	output("{\"group\":\"%s\",\"deviceType\":%d}\n", name, 1);
	//rssiGetHandler(nodeName);
#if 0
	output("{\"emNeighborTable\":\"6628\",\"set\":["
	       "{\"name\":\"8523\",\"addr\":8523,\"rssi\":-37},"// 39 chars per
	       "{\"name\":\"ad7d\",\"addr\":ad7d,\"rssi\":-42},"
	       "{\"name\":\"d4df\",\"addr\":d4df,\"rssi\":-43},"
	       "{\"name\":\"1b09\",\"addr\":1b09,\"rssi\":-45},"

	       "{\"name\":\"8523\",\"addr\":8523,\"rssi\":-37},"
	       "{\"name\":\"ad7d\",\"addr\":ad7d,\"rssi\":-42},"
	       "{\"name\":\"d4df\",\"addr\":d4df,\"rssi\":-43},"
	       "{\"name\":\"1b09\",\"addr\":1b09,\"rssi\":-45},"

	       "{\"name\":\"9bea\",\"addr\":9bea,\"rssi\":-57}"
	       "]}\n");
#endif
	}

void buttonHeld(int button)
	{
	string name = emResolveName(nodeAddr);
	printf("This address is: %x\n", nodeAddr);
	rssiGetHandler(nodeName);
	//rssiGet(rssiTable, nodeName);
	//dbSet(deviceType, 1);
	}

void initUI()
	{
	// the debug port is UART1, here we use UART0 as an alternate i/o port

	// create RX handler
	objectCreate(jsonEvent);
	OnEvent(jsonEvent, (HANDLER) jsonEventHandler);

	// init the uart link
	uartInit(0, serialRxReady);

	// for UART0 ints to work, we need the periferals on while sleeping
	setEventSleepMode(kSysSnooze);

	// capture the assert handler
	setAssert(__assertFailed);

	dbPublish(deviceType);
	//dbSet(renameValue, nodeAddr);
	}

