#include "EtherDATA.h"
#include "led.h"
#include "pCache.h"
#include "pindefs.h"
#include "sysgpio.h"
#include "string.h"
#include "Tag.ddl.h"

#include "battery.h"
#include "build.h"

int printf(const char *, ...);
void initBattery();
void initButton();
void initAccel();
void termAccel();
void initUI();
void initRSSI();
void termRSSI();
void initLED();
void initPTH();
void ledOn(word led, word delay);
void initPCACHE(void);
int delCompare(void *key, void *buf, SIZE_T size);
int init_dataLog(UInt16 frequency);
void initTimers();

#if 0
ExternTimer(configDeviceLEDTimer);

#define kConfigType       kFirstpCacheType + 3		// cached record type
#define ACCEL  0
#define HUMI   0
#define NEIGHBORTABLE   0

/****************************************************************************/
/*                      configValue DATA OBJECT                             */
/*      Drivers: [0] - NeighborTable (initRssi)
                 [1] - Temp/Humi Service (initHumi)
                 [2] - Accelerometer (initAccel)
*****************************************************************************/
void (*initValue[3])() = {initRSSI, initHumi, initAccel};
void (*termValue[3])() = {termRSSI, termHumi, termAccel};
STATUS configRequest(OBJECT o)
	{
	dbValue configValue = (dbValue)o;
	string data;
	char newData[5];
	byte request = 0;

	memset(newData, 0, sizeof(newData));

	dbGet(configValue, &data);
	strnbuf(5, newData, 0, "%s", data);

	memcpy(&request, &newData[4], sizeof(request));
	memset(&newData[4], 0, 1);

	strcpy(data, newData);

	if (strcmp("0000", newData))
		{
		printf("my new name is: %s\n", data);
		dbRename(nodeAddr, data);
		}

	if (request >= '0')
		{
		request = request - '0';
		}

	delChunk(kConfigType, NULL, delCompare);       // delete old threshold value from pCache
	newChunk(kConfigType, &request, 1);            // save new threshold value in pCache

	for (int i = 0; i < 3; i ++)
		{
		if (request % 2)
			{
			initValue[i]();
			}
		else
			{
			termValue[i]();
			}
		request = request / 2;
		}

	cmStartTimer(configDeviceLEDTimer);

	return 0;
	}
#endif

void tagSyncLost(byte state)
	{
	debug("Tag.syncLost: sync state=%u!\n", state);
	}

int main(int argc, char **argv)
	{
	(void) argv;
	(void) argc;
	print("%s\n\n", _versionStr);

	// check the battery level first!
	if (batterySample() < 2150)
		while (1)
			{
			ledOn(LED_RED, 100);
			cmDelayMs(500);
			}

	// these need to happen before dbInit
	setMeshWindow(128);
//	setMeshInterval(10000);

	// see if we've been given a name as part of the config
	dbInit(sysGetEnv("emNodeName"));

	emSetCntl(emMeshPower, 15);						// max RF power
	emSetCntl(emMeshChannel, 15);						// EtherSPY currently only deals with ch15
	emSetCntl(emSyncLostCallback, tagSyncLost);	// called whenever sync state changes

	emMeshStart();

	printf("Node %0x started as %s \"%s\"\n\n", nodeAddr, argv[0], nodeName);

#if 0
	UInt8 *temp;
	byte request;

	initPCACHE();
	initButton();
	initLED();
	initBattery(); // delay 30 ms before start timer
	cmDelayMs(750);
	initUI();

#if NEIGHBORTABLE
	initRSSI();
	//sysDelayMs(1000);
#endif

#if ACCEL
	initAccel();
	//sysDelayMs(1000);
#endif

#if HUMI
	initHumi();
	//sysDelayMs(1000);
#endif

	if ((temp = locChunk(kConfigType, NULL, delCompare)) == NULL);
	else
		{
		memcpy(&request, temp, 1);
		for (int i = 0; i < 3; i ++)
			{
			if (request % 2)
				{
				initValue[i]();
				}
			else
				{
				termValue[i]();
				}
			request = request / 2;
			}
		}

	//initHumi();
	//initAccel();
	//init_dataLog(0);


	dbPublish(logData);
	//ledBlink(redFlashPattern);
	ledOn(LED_RED, 200);
	//initAccel();
#endif

	initPTH();	// start PTH sampling

	// wait for events
	WaitEvent(0, 0);// never returns
	}
