#include "EtherDATA.h"

#include "pCache.h"
#include "pindefs.h"
#include "sysgpio.h"
#include "string.h"
#include "Tracker.ddl.h"

extern UInt16 sync_interval = 10000;
extern UInt16 groupResetCounter;
#define kConfigType       kFirstpCacheType + 3		// cached record type
#define ACCEL  0
#define HUMI   0
#define NEIGHBORTABLE   0

int printf(const char *, ...);

#if 0
void initBattery();
void initButton();
void initAccel();
void termAccel();
void initUI();
void initRSSI();
void termRSSI();
void initLED();
void initHumi();
void termHumi();
void ledBlink(Pattern pattern);
void initPCACHE(void);
int delCompare(void *key, void *buf, SIZE_T size);
void init_dataLog(void);
int start_dataLog(UInt16 frequency);
void initTimers();
void ledOn(word led, word delay);
void Sensor_Pwr(bool option);
ExternTimer(configDeviceLEDTimer);
void start_Humi();
void init_syncLost();

/****************************************************************************/
/*                      configValue DATA OBJECT                             */
/* Format:[4-bytes for new name][1 - byte for peripherals]                  */
/* Example: "Adam7"                                                         */
/*      Drivers: [0] - NeighborTable (initRssi)
                 [1] - Temp/Humi Service (initHumi)
                 [2] - Accelerometer (initAccel)
*****************************************************************************/
void (*initValue[3])() = {initRSSI, start_Humi, initAccel};
void (*termValue[3])() = {termRSSI, termHumi, termAccel};
STATUS configRequest(OBJECT o)
	{
	dbValue configValue = (dbValue)o;
	string data;
	string newName;
	char newData[5];
	byte request = 0;

	memset(newData, 0, sizeof(newData));

	dbGet(configValue, &data);
	strnbuf(5, newData, 0, "%s", data);

	memcpy(&request, &newData[4], sizeof(request));
	memset(&newData[4], 0, 1);

	strcpy(data, newData);

	if (strcmp("0000", newData))
		{
		printf("my new name is: %s\n", data);
		dbRename(nodeAddr, data);
		}

	if (request >= '0')
		{
		request = request - '0';
		}

	if ((locChunk(kConfigType, NULL, delCompare)) == NULL);
	else
		{
		for (int i = 0; i < 3; i ++)
			{
			termValue[i]();
			}
		}

	delChunk(kConfigType, NULL, delCompare);       // delete old threshold value from pCache
	newChunk(kConfigType, &request, 1);            // save new threshold value in pCache

	for (int i = 0; i < 3; i ++)
		{
		if (request % 2)
			{
			initValue[i]();
			}
		else
			{
			termValue[i]();
			}
		request = request / 2;
		}

	cmStartTimer(configDeviceLEDTimer);

	return 0;
	}


void deviceTypeHandler(string group, dbValue ref, void *value)
	{
	groupResetCounter = 0;
	ledOn(LED_GRN, 50);
	printf("DT*%s\n", group);
	}
#endif

void meshLost(byte state)
	{
	debug("Tracker.syncLost: sync state=%u!\n", state);
	}

int main(int argc, char **argv)
	{
	(void) argv;
	(void) argc;

#if 0
	/* Define when BAT values are finalized. This is designed to prevent device from joining mesh if battery is low*/
	int value = adc(0x00 | 0x30 | 0x0f) >> 3; // VDD/3 ref
	if (value < 2150)
		while(1)
			{
			ledBlink(redFlashPattern);
			sysDelayMs(250);
			}
#endif

	// these need to happen before dbInit
	setMeshWindow(128);
	setMeshPendCount(3);
//	setMeshInterval(10000);
	setMeshInterval(sync_interval);

	// see if we've been given a name as part of the config
	dbInit(sysGetEnv("emNodeName"));

	emSetCntl(emMeshPower, 15);						// max RF power
	emSetCntl(emMeshChannel, 15);						// EtherSPY currently only deals with ch15
	emSetCntl(emSyncLostCallback, meshLost);		// called whenever sync state changes

	emMeshStart();
	printf("Node %0x started as %s \"%s\"\n\n", nodeAddr, argv[0], nodeName);

#if 0
	UInt8 *temp;
	byte request;

	initPCACHE();
	initButton();
	initLED();
	initBattery(); // delay 30 ms before start timer
	cmDelayMs(500);
	initUI();

	//initHumi();
	//start_Humi();
	//initRSSI();

	if ((temp = locChunk(kConfigType, NULL, delCompare)) == NULL);
	else
		{
		memcpy(&request, temp, 1);
		for (int i = 0; i < 3; i ++)
			{
			if (request % 2)
				{
				initValue[i]();
				}
			else
				{
				termValue[i]();
				}
			request = request / 2;
			}
		}

	//initAccel();
	init_dataLog();
	//start_dataLog(0);

	dbPublish(logData);

	//GPIO.SET(ACC_ON, 1);
	ledOn(LED_RED, 100);
	//dbSubscribe("*", deviceType, (HANDLER)deviceTypeHandler);
#endif

	// wait for events
	WaitEvent(0, 0);// never returns
	}
