#include "EtherDATA.h"
#include "pCache.h"
#include "kNode.ddl.h"
#include "PHT Drivers.h"
#include "string.h"
#include "hal_i2c.h"

#include "TCA6416A.h"

int printf(const char *, ...);
double calcHumRel(UInt16 rawH);
double calcHumTmp(UInt16 rawT);
UInt8 HalI2CWrite(UInt8 len, UInt8 *pBuf);
void timerX(TIMER t, dbValue ref, UInt16 time);
UInt16 humi_sample_counter;
UInt16 humi_timer_counter;
double calcTemp();
double calcPres();
double *calcTempPress();
void Sensor_Pwr(bool option);
void Sensor_Select(int sensor);
int humiState = 0;
int TempHumid[3];
int delCompare(void *key, void *buf, SIZE_T size);
UInt8 D1[24];
UInt8 D2[24];

// Length of sensor data in bytes
#define HUMIDITY_DATA_LEN               4
#define kTempType         kFirstpCacheType + 1		// cached record type
#define kHumiType         kFirstpCacheType + 2		// cached record type
#define kHumiSampleInterval 	TICKS(30000)	// ms
//#define kHumiSampleInterval 	TICKS(5000)	// ms

int *sampleHumiTemp() // function to sample temp, humi, and pressure from sensor
	{
	UInt8 hData[HUMIDITY_DATA_LEN];
	unsigned long pData;
	unsigned long tData;
	//long Temp;
	//long Pressure;
	UInt16 humid_MSB;
	UInt16 humid_LSB;
	UInt16 rawHumid;
	static int data[3];
	double rh = 0.0;

	// need the i/o expander to stay awake
	waitExpander();

	SensorInit();
	cmDelayMs(20);

	humi_sample_counter = 0;

	while (humiState < 2)
		{
		HumiExecMeasurementStep(humiState);
		humiState++;
		}

	if (humiState == 2)
		{
		if (HumiReadMeasurement(hData))
			{
			humid_LSB = hData[2];
			humid_MSB = hData[3] << 8;
			rawHumid = (humid_LSB | humid_MSB);
			rh = calcHumRel(rawHumid);
			}
		humiState = 0;
		}

	PressureMeasurementStep();

	PresReadMeasurement(&pData);

	TempMeasurementStep();

	TempReadMeasurement(&tData);

	// done with i/o expander
	postExpander();

	double values[2];
	memcpy(values, calcTempPress(), sizeof(values));
	debug("P=%d.%u\n", (int) values[1], (word) ((values[1] - (int) values[1]) * 100));
	debug("H=%d.%u\n", (int) rh, (word) ((rh - (int) rh) * 100));
	debug("T=%d.%u\n", (int) values[0], (word) ((values[0] - (int) values[0]) * 100));

	data[0] = (int) values[0];
	data[1] = (int) rh;
	data[2] = (int) values[1];

	//Temp = values[0];
	//Pressure = values[1];

	// HalI2CDisable();
	return data;
	}

// define a humi sample timer
DefineTimer(humiSampleTimer);
void humiSampleTimerHandler(TIMER e, dbValue ref, UInt16 time)
	{
//	string desc;
//	string unit;

	humi_sample_counter ++;
	humi_timer_counter ++;

	if (humi_sample_counter >= (10))//30 = every 15 min
		{
		humi_timer_counter = 0;

		// sample the data
		dbSet(humiValue, sampleHumiTemp());

	//	dbGetAttribute(ref, humiValueEnum.units, &unit);
	//	dbGetAttribute(ref, humiValueEnum.description, &desc);
		dbGet(humiValue, TempHumid);
		printf("HTP:%d,%d,%d\n", TempHumid[0], TempHumid[1], TempHumid[2]);
		}
	}


void initHumi() // creates sample timer, resets counters, reads PROM data
	{
	humi_sample_counter = 0;
	humi_timer_counter = 0;

	// set up and start the humidity sampling timer
	objectCreate(humiSampleTimer, kIntervalTimer, kHumiSampleInterval);// once per second
	OnEvent(humiSampleTimer, (HANDLER) humiSampleTimerHandler);

	// need the i/o expander to stay awake
	waitExpander();

	// init the driver
	SensorInit();

	for (int i = 0; i < 8; i ++) // upon reset we need to retreive PROM data from sensor
		{
		Sensor_Prom(i);
		//cmSetTimer(humiDelayTimer, TICKS(20));
		//cmStartTimer(humiDelayTimer);
		}
	//humi_sample_counter = 30;
	//humiSampleTimerHandler(humiSampleTimer, humiValue, 0);

	// done with i/o expander
	postExpander();
	}

void start_Humi() // publishes humiValue and starts sample timer
	{
	humi_sample_counter = 0;
	humi_timer_counter = 0;

	// publish the data
	dbPublish(humiValue);

	cmStartTimer(humiSampleTimer, humiValue, 1);
	}

void termHumi() // withdraws humiValue and stops sample timer
	{
	dbWithdraw(humiValue);
	cmStopTimer(humiSampleTimer);
	}
