#include <string.h>

#include "Koliada.h"
#include "event.h"

#include "EtherDATA.h"
#include "kNode.ddl.h"
#include "pCache.h"

#include "TCA6416A.h"
#include "sysgpio.h"
#include "pindefs.h"
#include "LIS2DH.h"


#define kAccelSampleInterval 	TICKS(1000)	// once per 5 second
#define USE_ACCEL 1
#define DATA_RATE		3			// ODR 2=10Hz, 3=25Hz, 4=50Hz See Table 28

/* External Declarations */
int printf(const char *, ...);
void accLPM(void);
void button2EventHandler(EVENT e, word ifg, word ier);
void MemledOn(word led, word delay);
void TopledOn(word delay);
void lis2dh_init(void);

/* static variables */
byte randomByte();
UInt8 acc_threshold = 0;
UInt8 original_count = 0;
byte accel_mode = 0;
UInt8 movementCounter = 0;
bool rssiTimerUpdate = 0;
UInt8 DOOR_FLAG = 0;

DefineTimer(publishLimitTimer); // this is called after 'x' delay to rePublish accelValue (limits the amount of publishes for accel)
void publishLimitHandler(TIMER t)
	{
	dbPublish(accelValue);
	//PostEvent(publishLimitTimer);
	}

DefineTimer(movementTimer);// movement timer designed to increase the frequency of neighbor table publishing if item has been moved
void movementHandler(TIMER t, UInt16 time)
	{
	movementCounter ++;
	if (movementCounter >= (2  * time))
		{
		movementCounter = 0;
		//cmStopTimer(rssiSampleTimer);
		//cmStartTimer(rssiSampleTimer, rssiTable, 0);
		rssiTimerUpdate = 0;
		}
	}

DefineTimer(doorTimer); //Used as part of the ACCEL door logic
void doorHandler(TIMER t)
	{
	DOOR_FLAG = 0;
	}

void configDrop() // configure the ACCEL with Drop settings
	{
	accel_mode = DROP;                              // update accel_mode variable
	HalI2CDisable();                                // ensure i2c is not talking to peripherals
	selectI2C(LIS2DH12_ADDR, i2cClock_267KHZ);      // select accel i2c slave address and i2c frequency
	//sysDelayMs(5);

	// set up accel
	i2cTx(LIS2DH_CTRL_REG1, (DATA_RATE << 4) + 0x0F);	// normal mode all-axis enabled
	i2cTx(LIS2DH_CTRL_REG2, 0x00);
	i2cTx(LIS2DH_CTRL_REG3, 0x80);                  //click enabled on int1
	i2cTx(LIS2DH_CTRL_REG4, 0x20);                  // 4G -0x20
	i2cTx(LIS2DH_CTRL_REG5, 0x00);
	//i2cTx(LIS2DH_CTRL_REG6, 0x00);                //interrupt active high
	i2cTx(LIS2DH_CTRL_REG6, 0x02);                  //interrupt active low

	/* check lis2dh data sheet for specifications on registers, found this configuration online to be for a 'drop' profile */
	i2cTx(LIS2DH_TIME_LIMIT, 0x08);//1
	i2cTx(LIS2DH_TIME_WINDOW, 0x0a);
	i2cTx(LIS2DH_CLICK_THS, 0x7f);//7
	i2cTx(LIS2DH_CLICK_CFG, 0x15);		        // x & y & z axis single click
	}


void configDoor() // configure the ACCEL with Door settings
	{
	accel_mode = DOOR;                              // update accel_mode variable
	HalI2CDisable();                                // ensure i2c is not talking to peripherals
	selectI2C(LIS2DH12_ADDR, i2cClock_267KHZ);      // select accel i2c slave address and i2c frequency
	//sysDelayMs(5);

	// set up accel
	i2cTx(LIS2DH_CTRL_REG1, (DATA_RATE << 4) + 0x0C);	 // normal mode all-axis enabled
	i2cTx(LIS2DH_CTRL_REG2, 0x00);
	i2cTx(LIS2DH_CTRL_REG3, 0x40);
	i2cTx(LIS2DH_CTRL_REG4, 0x00);                     // 2G
	i2cTx(LIS2DH_CTRL_REG5, 0x00);
	//i2cTx(LIS2DH_CTRL_REG6, 0x00);                   // interrupt active high
	i2cTx(LIS2DH_CTRL_REG6, 0x2A);                     //interrupt active low

	/* int 1 - more sensitive, but has min duration*/
	i2cTx(LIS2DH_INT1_THS, 0x02);		// 32LSB*15.625mg/LSB=0.5g
	i2cTx(LIS2DH_INT1_DURATION, 0x04);	// Duration= 10LSB*(1/10Hz) = 1sec
	i2cTx(LIS2DH_INT1_CFG, 0x70);		// Direction detection on z-axis

	/* int 2 - less sensitive, no min duration */
	i2cTx(LIS2DH_INT2_THS, 0x05);		// 32LSB*15.625mg/LSB=0.5g
	i2cTx(LIS2DH_INT2_DURATION, 0x00);	// Duration= 10LSB*(1/10Hz) = 1sec
	i2cTx(LIS2DH_INT2_CFG, 0x70);		// Direction detection on z-axis
	}

void enableAccel() // set ACCEL_ON GPIO high
	{
	GPIO.SET(ACC_ON, 1);
	}

void disableAccel() // set ACCEL_ON GPIO low
	{
	GPIO.SET(ACC_ON, 0);
	}

void termAccel() // set ACCEL_ON GPIO low and stop publishing ACCEL_value
	{
	//dbWithdraw(accelValue);
	disableAccel();
	}

#if 1
#define kAccelType     	   kFirstpCacheType		// cached record type
#define DROP 1
#define DOOR 2

int delCompare(void *key, void *buf, SIZE_T size);
extern byte accel_mode;// accel.c

STATUS accConfigChanged(OBJECT o) // Accel Config Handler! Called when Devices's accelConfig dataObject is written to
	{
	byte request;
	dbValue accelConfig = (dbValue)o; // set accelConfig data object to local input parameter 'o'

	dbGet(accelConfig, &request); // get accelConfig value and store it in 'request' variable
	if (request >= '0') // if 'request' value is in ASCII
		{
		request = request - '0'; // subtract '0' so that request is an int
		}
	debug("accel Config: %d\n", request);

	delChunk(kAccelType, NULL, delCompare);                         // delete old threshold value from pCache
	newChunk(kAccelType, &request, sizeof(request));            // save new threshold value in pCache
	memcpy(&accel_mode, &request, sizeof(request));
	switch (request)
		{
		case DROP:
				{
				//configDrop();
				break;
				}
		case DOOR:
				{
				//configDoor();
				break;
				}
		default:
				{
				//configDoor();
				break;
				}
		}
	request += 2;

	dbSet(accelValue, request); // set accelValue to the new config value + 2 as part of a confirmation
	return 0;
	}
#else
#warning accelRequest not currently supported!
STATUS accConfigChanged(OBJECT o)
	{
	debug("%s\n", __FUNC__);
	return 0;
	}
#endif

DefineEvent(accInt1Event);
void accInt1Handler(EVENT e) // GPIO event handler when ACCEL_INT1 occurs
	{
//	byte source = 0;
	byte int1 = 0;
	byte int2 = 0;

	int1 = GPIO.GET(ACC_INT1);              // get GPIO state. ACC ints are active LOW
	int2 = GPIO.GET(ACC_INT2);              // get GPIO state. ACC ints are active LOW

	switch (accel_mode)
		{
		case DOOR:
				{
				if (!int2)                          // int 2 triggered (a short impulse along z axis)
					{
					DOOR_FLAG = 1;                    // set door flag
					cmStopTimer(doorTimer);           // make sure the door timer is not already running
					cmStartTimer(doorTimer);           // start doorTimer
					}
				if (!int1)                          // int 1 triggered (longer movement along z - axis)
					{
					if (DOOR_FLAG)                    // if flag has been set
						{
						if (device_state == LOGGING_MODE) // if in logging mode, then save event in flash
							{
							debug("accel in logging mode \n");
							//addAccel_dataLog();***
							}
						DOOR_FLAG = 0;                     // reset flag
						//source = i2cRx(LIS2DH_INT1_SOURCE); // for debug purposes you can check to see which int source caused trigger
						//debug("DOOR INT: %x\n", source);
						dbSet(accelValue, accel_mode);    // publish value
						MemledOn(LED_GRN, 50);            // visual indication

						dbWithdraw(accelValue);           // stop publishing
						cmSetTimer(publishLimitTimer, TICKS(30000)); // set limit timer to 'x' ms. NOTE: This needs to be equal or greater than mesh sync interval
						cmStartTimer(publishLimitTimer); // start limit timer

						debug("accel mode:%d\n", accel_mode);
						GPIO.IFG(ACC_INT1, 0); // reset int IFG
						GPIO.IEN(ACC_INT1, 1); // re-enable interrupts
						GPIO.IFG(ACC_INT2, 0); // ''
						GPIO.IEN(ACC_INT2, 1); // ''
						}
					}
				break;
				}

		case DROP:
				{
				if (!int1) // not currently implemented
					{
					movementCounter = 0;
					cmStopTimer(movementTimer);
					cmStartTimer(movementTimer, 2);
					if (!rssiTimerUpdate)
						{
						//cmStopTimer(rssiSampleTimer);
						//cmStartTimer(rssiSampleTimer, rssiTable, 0);
						rssiTimerUpdate ++;
						}
					}
				if (!int2)
					{
					if (device_state == LOGGING_MODE)
						{
						//ledOn(LED_RED, 100);
						debug("accel in logging mode \n");

						//addAccel_dataLog();***********                // be sure to un-commment this out when dataLogging is used!
						}
					else
						{
						TopledOn(50);
						//ledBlink(redFlashPattern);

						debug("accel mode:%d\n", accel_mode);
						dbSet(accelValue, accel_mode);
						dbWithdraw(accelValue);
						}
					GPIO.IFG(ACC_INT2, 0);
					GPIO.IEN(ACC_INT2, 1);

					cmSetTimer(publishLimitTimer, TICKS(10000));
					cmStartTimer(publishLimitTimer);
					}
				break;
				}
		}

	GPIO.IFG(ACC_INT1, 0);
	GPIO.IEN(ACC_INT1, 1);
	GPIO.IFG(ACC_INT2, 0);
	GPIO.IEN(ACC_INT2, 1);
	}

byte lis2dh_getId(void) // get ACCEL internal ID.
	{
	selectI2C(LIS2DH12_ADDR, i2cClock_267KHZ); // select IO expander as I2C slave
	return i2cRx(LIS2DH_WHO_AM_I);
	}

void initAccel() // create timers and publish accelValue (DEFAULT = DROP)
	{
	GPIO.SET(ACC_ON, 1);
	// set up to handle accel interrupts
	objectCreate(accInt1Event);
	OnEvent(accInt1Event, (HANDLER) accInt1Handler);

	//objectCreate(accInt2Event);
	//OnEvent(accInt2Event, (HANDLER) accInt2Handler);

	byte test = lis2dh_getId();
	// capture the interrupts to triger the event
	setEventIRQ(PORT_IRQ(PIN_PORT(ACC_INT1)), accInt1Event);

	dbPublish(accelValue);

	objectCreate(movementTimer, kIntervalTimer, TICKS(30000));
	OnEvent(movementTimer, (HANDLER) movementHandler);

	objectCreate(doorTimer, kSingleShotTimer, TICKS(300));
	OnEvent(doorTimer, (HANDLER) doorHandler);

	objectCreate(publishLimitTimer, kSingleShotTimer, 0);
	OnEvent(publishLimitTimer, (HANDLER) publishLimitHandler);

	configDrop(); // configure ACCEL to be DROP

	// enable interrupts for the buttons
	GPIO.IFG(ACC_INT1, 0);
	GPIO.IEN(ACC_INT1, 1);

	// enable interrupts for the buttons
	GPIO.IFG(ACC_INT2, 0);
	GPIO.IEN(ACC_INT2, 1);

	//disableAccel();
	}
