#include "EtherDATA.h"
#include "TCA6416A.h"
#include "kNode.ddl.h"
#include "sysGpio.h"
#include "adc.h"
#include "led.h"
#include "buzzer.h"
#include "pindefs.h"
int printf(const char *, ...);

#define kHoldTime		 TICKS(1500)
#define kPressTime	 TICKS(250)

void buttonHeld(int button);	// ui.c
void buttonPressed(int button);	// ui.c
void buttonLongHold(int button);
void MemledOn(word led, word delay);
byte buttonPressedCount;
void accInt2Handler(EVENT e);

static bool isPressed(int button)
	{
	return (button == IOX_INT ? (~readExpander(PORT0_IN) & (BUTTON_3)) : (GPIO.GET(button) == 0));
	}

DefineTimer(buttonHeldTimer);
void buttonHeldHandler(TIMER t, int button)
	{
	if (button == BUTTON_1) // check to see if there was a 'two button press'
		{
		if (isPressed(IOX_INT))
			{
			printf("b:1 & 3\n");
			button = 1;
			}
		buttonHeld(button);
		}
	else if (button == IOX_INT) // check to see if there was a 'two button press'
		{
		if (isPressed(BUTTON_1))
			{
			printf("b:3 & 1\n");
			button = 1;
			}
		buttonHeld(button);
		}
	else
		{
		buttonHeld(button);
		}

	GPIO.IFG((IOX_INT), 0);
	GPIO.IEN((IOX_INT), 1);
	GPIO.IFG((BUTTON_1), 0);
	GPIO.IEN((BUTTON_1), 1);
	GPIO.IFG((BUTTON_2), 0);
	GPIO.IEN((BUTTON_2), 1);
	}

DefineTimer(buttonDebounceTimer);
void buttonDebounceTimerHandler(TIMER t, int button) // occurs after debounce timer has occured
	{
	if (isPressed(button))
		{
		// button is still pressed - see if it is a 'hold'
		cmStartTimer(buttonHeldTimer, button);		// add the button as an arg
		}
	else
		{
		// must be a press
		buttonPressed(button);
		GPIO.IFG(button, 0);
		GPIO.IEN(button, 1);
		}
	}

DefineEvent(button1Event);
void button1EventHandler(EVENT e, word ifg, word ier) // occurs when Port 2 interrupt occurs
	{
	if (ifg & (0x01 | 0x02)) // check to see if its P2_0 or P2_1 (button 1 or button 2)
		{
		if (ifg & 0x01) // if button 1
			{
			cmStartTimer(buttonDebounceTimer, BUTTON_1);		// add the button as an arg and start timer
			}
		else // must be button 2
			{
			cmStartTimer(buttonDebounceTimer, BUTTON_2);		// add the button as an arg and start timer
			}
		}
	}

void button3Event() // button 3 event [handled by IO expander (see TCA6416A.c)]
	{
	cmStartTimer(buttonDebounceTimer, IOX_INT);
	}

void initButton() // create timers, set Port 2 IRQ to button 1 [button 1 and 2 are on Port 2, therefore only one IRQ can be used ( all are shared on Port 2)]
	{
	// button debounce timer
	objectCreate(buttonDebounceTimer, kSingleShotTimer, kPressTime);
	OnEvent(buttonDebounceTimer, (HANDLER) buttonDebounceTimerHandler);

	// button held timer
	objectCreate(buttonHeldTimer, kSingleShotTimer, kHoldTime);
	OnEvent(buttonHeldTimer, (HANDLER) buttonHeldHandler);

	// create a button 1 event
	objectCreate(button1Event);
	OnEvent(button1Event, (HANDLER) button1EventHandler);

	// capture BUTTON interrupt to triger the event
	// note: it is up to the developer to manage overlapping pin port IRQs!
	setEventIRQ(PORT_IRQ(PIN_PORT(BUTTON_1)), button1Event);

	// enable interrupts for the buttons
	GPIO.IFG(BUTTON_1, 0);
	GPIO.IEN(BUTTON_1, 1);

	// enable interrupts for the buttons
	GPIO.IFG(BUTTON_2, 0);
	GPIO.IEN(BUTTON_2, 1);
	}
