#include "buzzer.h"
#include "sysgpio.h"
#include "pindefs.h"
#include "event.h"
// a note is a frequency played for a duration
// a tune is a collection of notes repeated N times
// note frequency is defined by the prescaler and compare values

// Defined PWM Frequency Settings
//                     /----- prescaler value
//                   /     /- compare value
//                --|-  --|-
//#define LOFREQ		0x0C, 0xF9
#define HIFREQ		0x0C, 0x0E
//#define HIFREQ		0x08, 0xF9
#define LOFREQ		0x0C, 0x12
#define NILFREQ	        0x00, 0x00

////////////////////////////////////////////////////////////////////////////////

Note beepLoNotes[] =
	{
		{ LOFREQ, 200, {1} },
	};

TuneDecl beepLoDecl =
	{
	1,					// repeats
	sizeof(beepLoNotes) / sizeof(Note),	// note count
	beepLoNotes,
	};
Tune beepLo = &beepLoDecl;

////////////////////////////////////////////////////////////////////////////////

Note beepHiNotes[] =
	{
		{ HIFREQ, 200, {1} },
	};

TuneDecl beepHiDecl =
	{
	1,					// repeats
	sizeof(beepHiNotes) / sizeof(Note),	// note count
	beepHiNotes,
	};
Tune beepHi = &beepHiDecl;

////////////////////////////////////////////////////////////////////////////////

Note beepNILNotes[] =
	{
		{ NILFREQ, 100, {0} },
	};

TuneDecl beepNILDecl =
	{
	0,					// repeats
	sizeof(beepNILNotes) / sizeof(Note),	// note count
	beepNILNotes,
	};
Tune beepNIL = &beepNILDecl;

////////////////////////////////////////////////////////////////////////////////

Note msgNotes[] =
	{
		{ LOFREQ,  200, {1} },
		{ HIFREQ,  200, {1} },
		{ NILFREQ, 150, {0} },
		{ LOFREQ,  200, {1} },
		{ HIFREQ,  200, {1} },
		{ NILFREQ, 500, {0} },
	};

TuneDecl msgTuneDecl =
	{
	1,				// repeats
	sizeof(msgNotes) / sizeof(Note),	// note count
	msgNotes,
	};
Tune msgTune = &msgTuneDecl;

////////////////////////////////////////////////////////////////////////////////

TuneDecl initTuneDecl =
	{
	1, // repeats
	sizeof(msgNotes) / sizeof(Note),	// note count
	msgNotes,
	};
Tune initTune = &initTuneDecl;

////////////////////////////////////////////////////////////////////////////////

TuneDecl findTuneDecl =
	{
	30,				// repeats
	sizeof(msgNotes) / sizeof(Note),	// note count
	msgNotes,
	};
Tune findTune = &findTuneDecl;

////////////////////////////////////////////////////////////////////////////////

static void buzzerInit(void)
	{
	//chainEventSleepNotice = setEventSleepNotice(eventSleepNotice);
	// setup T3 as buzzer PWM
	//PERCFG |= 0x20;             // Timer 3 Alternate location 2
	//P0DIR |= 0x40;              // P1_6 = output
	P0SEL |= 0x40;              // Peripheral function on P1_6

	T1CCTL4 &= ~0x40;           // Disable channel 0 interrupts
	T1CCTL4 |= 0x04;            // Ch0 mode = compare
	T1CCTL4 |= 0x10;            // Ch0 output compare mode = toggle on compare
	T1CC4H = 0x00;
	T1CC4L = 0x00;
	}

// the tune timer
DefineTimer(tuneTimer);
static void tuneTimerHandler(TIMER e, Tune tune, word repeats, word note)
	{
	// stop pwm
	T1CTL &= ~0x03;

	// program the note
	T1CTL &= ~0x0C;
	T1CTL = 0x00;
	//T1CCTL4 = (T1CCTL4 & ~0x23) | 0x1C;
	T1CTL |= tune->note[note].prescaler;
	T1CC0L = tune->note[note].compare;
	T1CC0H = 0x00;

	if (note < tune->noteCount)
		{
		if (tune->note[0].enable)
			// Start pwm
			T1CTL |= 0x03;              // Timer 1 mode = 3 - Up/Down

		// set timer for this note
		cmSetTimer(e, tune->note[note].duration);

		// set args for the next note
		cmStartTimer(e, tune, repeats, ++note);
		}
	else if (repeats)
		{
		// start over
		tuneTimerHandler(e, tune, --repeats, 0);
		}
	else
		{
		cmStopTimer(e);
		GPIO.SET(BUZZER, 0);
		cmDelayMs(100);
		stopBuzzer();
		T1CTL &= ~0x03;
		}
	}

void playTune(Tune tune)
	{
	buzzerInit();
	// start the tune
	tuneTimerHandler(tuneTimer, tune, tune->repeats - 1, 0);
	}

Tune requestList[] =
	{
	&findTuneDecl,
	&beepLoDecl,
	&beepHiDecl,
	&msgTuneDecl,
	};
#define sizeofRequestList (sizeof(requestList)/sizeof(Tune))

STATUS playRequest(OBJECT o, bool isArray, byte index)
	{
	dbValue buzzer = (dbValue)o;
	byte request;

	dbGet(buzzer, &request);
	if (request >= '0')
		{
		request = request - '0';
		}
	//debug(">>%s - request %u\n", __func__, request);
	if (request < sizeofRequestList)
		playTune(requestList[request]);

	return 0;
	}

#define DEBUG_BUZZER 0
#if DEBUG_BUZZER
// define a buzzer timer
DefineTimer(buzzerTimer);
void buzzerTimerHandler(TIMER e)
	{
	debug(">>%s\n", __func__);
	playTune(msgTune);
	debug("<<%s\n", __func__);
	}
#endif


void initBuzzer()
	{
	//buzzerInit();
	GPIO.SET(BUZZER, 0);
	// set up the Tune timer
	objectCreate(tuneTimer, kIntervalTimer, 0);
	OnEvent(tuneTimer, (HANDLER) tuneTimerHandler);

	// play the init sound
	//playTune(msgTune);
	}

void stopBuzzer()
	{
	GPIO.SET(BUZZER, 0);
	T1CTL &= ~0x03;
	}
