#include "EtherDATA.h"
#include "kNode.ddl.h"
#include "pCache.h"
#include "pindefs.h"
#include "hal_humi.h"
#include "string.h"
#include "led.h"
#include "flash.h"

#define kdataLogType                    kFirstpCacheType + 4
#define kdataLogCounterType             kFirstpCacheType + 5
#define HUMIDITY_DATA_LEN               4

#define PCACHE_ADDR ((ROMPTR) 0xB000)
#define NUM_PAGES 4
#define PAGE_SIZE 2048
#define NUM_RESERVE 1

UInt16 dataLog_counter;
UInt16 datalog_Handler_counter;
UInt16 push_counter;
extern UInt16 TempHumid[2];
extern UInt16 sample_number;
int *sampleHumiTemp();
int printf(const char *, ...);
int delCompare(void *key, void *buf, SIZE_T size);
int dataLogCompare(void *key, void *buf, SIZE_T size);
byte device_state;
boolean checkFlush(wyde type, void *buf, SIZE_T size);
void MemledOn(word led, word delay);
void *addr = (void *) PCACHE_ADDR;

DefineTimer(dataLogTimer);
DefineTimer(pushLogTimer);
ExternTimer(humiSampleTimer);
ExternTimer(rssiSampleTimer);
DefineTimer(awakeTimer);
void humiSampleTimerHandler(TIMER e, dbValue ref, UInt16 time);

UInt16 *read_dataLog(long base_address, UInt16 sample_number) // read dataLog values from FLASH
	{
	UInt16 static rx_buf[4];
	//UInt16 test[8];
	memset(rx_buf, 0xFF, sizeof(rx_buf));
	long addr_shift = base_address + (sample_number - 1) * 8;

	sysFlashFetch(FlashAddr(addr_shift), (byte *)rx_buf, 2);
	//sysFlashFetch(FlashAddr(addr_shift-4), (byte *)test, 4);

	return rx_buf;
	}

void addAccel_dataLog() // Add ACCEL Event in pCache
	{
	if ((locChunk(kdataLogType, &dataLog_counter, dataLogCompare)) == NULL)
		{
		UInt16 append;
		append = dataLog_counter;
		newChunk(kdataLogType, &append, sizeof(append));            // save new threshold value in pCache
		}
	}

void store_dataLog(long base_address, UInt16 *buf, UInt16 sample_number) // save dataLog value in FLASH
	{
	UInt16 data[4];

	memcpy(&data, buf, sizeof(data));
	long addr_shift = base_address + (sample_number - 1) * 8;
	sysFlashWrite(FlashAddr(addr_shift), (byte *)data, 2);
	}


void push_dataLog() // push dataLog Values
	{
	//push_counter = sample_number;
	push_counter = dataLog_counter; // set push counter to last sample counter
	cmStartTimer(pushLogTimer); // start 'push' timer
	}

// Push Timer Handler
void pushLogTimerHandler(TIMER e)
	{
	static char data[18] = {PUSH, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	char buf[14]; // temp variable used to transfer buffer to string
	UInt16 flashData[4];
	if (push_counter == 0)
		{
		cmStopTimer(pushLogTimer);
		}
	else
		{
		//memset(data, '0', 18);

		memset(buf, '0', sizeof(buf));

		//temp = locChunk(kdataLogType, &push_counter, dataLogCompare);
		memcpy(&flashData, read_dataLog(0x26000, push_counter), sizeof(flashData));

		strnbuf(3, buf, 0, "%03d", push_counter); // SAMPLE # in string format
		memcpy(&data[1], buf, 3); // SAMPLE # in data
		memset(buf, '0', sizeof(buf)); // reset buf

		strnbuf(5, buf, 0, "%06d", flashData[0]); // TEMP in string format
		memcpy(&data[4], buf, sizeof(buf)); // TEMP in data
		memset(buf, '0', sizeof(buf)); // reset buf

		strnbuf(4, buf, 0, "%04d", flashData[1]); // HUMI in string format
		memcpy(&data[9], buf, 4); // HUMI in data
		memset(buf, '0', sizeof(buf)); // reset buf

		//UInt16 *temp;
		if ((locChunk(kdataLogType, &push_counter, dataLogCompare)) == NULL) // check to see if there was an ACCEL event in PCACHE registered for that sample number
			{
			memset(&flashData[3], 0, 2);
			}
		else
			{
			memset(&flashData[3], '1', 2);
			}
		strnbuf(1, buf, 0, "%d", flashData[3]); // ACCEL in string format
		memcpy(&data[13], buf, 1); // ACCEL in data
		memset(buf, '0', sizeof(buf)); // reset buf

		strnbuf(4, buf, 0, "%04d", flashData[2]); // PRESSURE in string format
		memcpy(&data[14], buf, 4); //PRESSURE in data

		/* ***** pCACHE PUSH FORMAT ***NOT USED ANYMORE *******
		strnbuf(3, buf, 0, "%03d", temp[2]);
		memcpy(&data[1], buf, 3);
		memset(buf, '0', sizeof(buf));
		strnbuf(5, buf, 0, "%06d", temp[0]);
		memcpy(&data[4], buf, 5);
		memset(buf, '0', sizeof(buf));
		strnbuf(4, buf, 0, "%04d", temp[1]);
		memcpy(&data[9], buf, 4);
		memset(buf, '0', sizeof(buf));
		strnbuf(1, buf, 0, "%d", temp[3]);
		memcpy(&data[13], buf, 1);
		memset(buf, '0', sizeof(buf));
		strnbuf(4, buf, 0, "%04d", temp[4]);
		memcpy(&data[14], buf, 4);*/

		dbSet(logData, data);
		MemledOn(LED_BOTH, 100);
		printf("%s \n", data);
		//printf("Flash Reading Temp/Humi[%d, %d]. Sample Number: %d. Accel %d. Pressure: %d\n", flashData[0], flashData[1], push_counter, flashData[3], flashData[2]);

		push_counter --;
		}
	}

void dataLogTimerHandler(TIMER e, UInt16 count) // logging timer when in Log Mode
	{
	datalog_Handler_counter ++;
	if (datalog_Handler_counter >= count)
		{
		datalog_Handler_counter = 0;
		//UInt16 static data[5];
		UInt16 static flash_data[4];

		dataLog_counter += 1;
		// memset(data, 0, sizeof(data));
		//memcpy(&data[0], sampleHumiTemp(), sizeof(data));
		memcpy(&flash_data[0], sampleHumiTemp(), sizeof(flash_data)); // sample temp/humi/pressure values
		//memset(&data[3], 0, 4);
		//memcpy(&data[4], &data[2], 4);
		//memcpy(&flash_data[3], &data[2], 4);
		memset(&flash_data[3], 0xFF, 2);
		//int length = sizeof(data);

		//memcpy(&data[2], &dataLog_counter, sizeof(dataLog_counter));
		printf("Temp/Humi [%d, %d]. Sample number: %d Pressure: %d\n", flash_data[0], flash_data[1], dataLog_counter, flash_data[2]); // debug statement


		store_dataLog(0x26000, flash_data, dataLog_counter); // store data Log values in FLASH

		//newChunk(kdataLogType, &data, sizeof(data));            // save new value in pCache

		delChunk(kdataLogCounterType, NULL, delCompare);
		newChunk(kdataLogCounterType, &dataLog_counter, 1);

		if (dataLog_counter >= 999) // max value
			{
			cmStopTimer(dataLogTimer); // Stop logging timer
			}
		}
	}

void awakeTimerHandler(void) // timer that periodically publishes 'AWAKE' value
	{
	char data[18];
	char buf[3];
	memset(data, '0', sizeof(data));
	strnbuf(sizeof(buf), buf, 0, "%03d", dataLog_counter);
	memcpy(&sample_number, &dataLog_counter, sizeof(dataLog_counter));
	memcpy(&data[1], buf, sizeof(buf));
	memset(data, AWAKE, 1); // Set mode bit as awake
	//memset(&data[1], dataLog_counter, 1);
	dbSet(logData, data); // Notify gateway that device is AWAKE from log
	}

int start_dataLog(UInt16 frequency) // reset all counters, start dataLog timer, stop Mesh Network
	{
	dataLog_counter = 0;
	datalog_Handler_counter = 0;
	push_counter = 0;
	cmStopTimer(humiSampleTimer);
	cmStopTimer(rssiSampleTimer);
	cmStartTimer(dataLogTimer, (2 * frequency));

	device_state = LOGGING_MODE;

	emMeshStop();

	return 1;
	}

void init_dataLog(void) // create timers, check to see if there are values currently stored
	{
	UInt16 *temp;
	//dataLog_counter = 0;
	datalog_Handler_counter = 0;
	push_counter = 0;

	if ((temp = locChunk(kdataLogCounterType, NULL, delCompare)) == NULL)
		{
		dataLog_counter = 0;
		}
	else
		{
		device_state = LOGGING_MODE;
		memcpy(&dataLog_counter, temp, sizeof(dataLog_counter));
		}

	objectCreate(dataLogTimer, kIntervalTimer, TICKS(2000));
	OnEvent(dataLogTimer, (HANDLER) dataLogTimerHandler);

	objectCreate(pushLogTimer, kIntervalTimer, TICKS(2000));
	OnEvent(pushLogTimer, (HANDLER) pushLogTimerHandler);

	objectCreate(awakeTimer, kIntervalTimer, TICKS(30000));
	OnEvent(awakeTimer, (HANDLER) awakeTimerHandler);

	sysFlashErase(FlashPage(0x26000));
	sysFlashErase(FlashPage(0x26800));
	sysFlashErase(FlashPage(0x27600));
	sysFlashErase(FlashPage(0x28400));

	dbPublish(logData);
	}

#if 0
/*
logData[Temp, Humid, Accel, Sample_Number / Frequency, mode]

MODES:
1 - Config - User selecting frequency
2 - Confirm/Start - Logger confirming frequency and initiating log
3 - Awake - Logger waking up, also stating the # of samples recorded.
4 - Ready - User ready to receive data
5 - Push - Logger Pushing data
6 - Recover - User missed 'X' sample
7 - Complete - User received all samples
*/
STATUS logRequest(OBJECT o) // logData HANDLER! Called everytime device's logData is written to
	{
	char request[18];
	memset(request, 0, sizeof(request));
	dbValue setlogData = (dbValue)o;
	dbGet(setlogData, &request);

	switch (request[0])
		{
		case CONFIG:
				{
				char reply[18];
				memset(reply, 0, sizeof(reply));
				memcpy(reply, request, sizeof(reply));
				memset(&reply[0], CONFIRMATION, 1); // Create a Confirmation Reply
				dbSet(logData, &reply); // Send gateway confirmation that device is in Logging Mode
				printf("reply\n");
				break;
				}

		case START:
				{
				device_state = LOGGING_MODE;
				char reply[18];
				UInt16 frequency = 0;
				memset(reply, 0, sizeof(reply));
				memcpy(reply, request, sizeof(reply));
				memset(&reply[0], CONFIRMATION, 1); // Create a Confirmation Reply
				//dbSet(logData, &reply); // Send gateway confirmation that device is in Logging Mode
				frequency = (request[3] - '0') * 10 + (request[4] - '0');
				printf("DL config Frequency = %d \n", frequency);
				start_dataLog(frequency); // start data logging
				break;
				}

		case READY:
				{
				printf("Starting to push\n");
				cmStopTimer(dataLogTimer); // Stop logging timer
				cmStopTimer(awakeTimer);
				push_dataLog();
				break;
				}

		case RECOVER:
				{
				printf("MISSED\n");
				UInt16 temp[4];
				char reply[18];
				char buf[14];
				UInt16 recover_packet_number = 0;
				memset(reply, '0', sizeof(reply));
				memset(&reply[0], MISSED, 1);
				recover_packet_number = (request[1] - '0') * 100 + (request[2] - '0') * 10 + (request[3] - '0');

				memcpy(&temp, read_dataLog(0x26000, recover_packet_number), sizeof(temp));
				//temp = locChunk(kdataLogType, &recover_packet_number, dataLogCompare);
				strnbuf(3, buf, 0, "%03d", recover_packet_number); // SAMPLE # in string format
				memcpy(&reply[1], buf, 3); // SAMPLE # in data
				memset(buf, '0', sizeof(buf)); // reset buf

				strnbuf(5, buf, 0, "%06d", temp[0]); // TEMP in string format
				memcpy(&reply[4], buf, sizeof(buf)); // TEMP in data
				memset(buf, '0', sizeof(buf)); // reset buf

				strnbuf(4, buf, 0, "%04d", temp[1]); // HUMI in string format
				memcpy(&reply[9], buf, 4); // HUMI in data
				memset(buf, '0', sizeof(buf)); // reset buf

				if ((locChunk(kdataLogType, &push_counter, dataLogCompare)) == NULL)
					{
					memset(&temp[3], 0, 2);
					}
				else
					{
					memset(&temp[3], '1', 2);
					}
				strnbuf(1, buf, 0, "%d", temp[3]); // ACCEL in string format
				memcpy(&reply[13], buf, 1); // ACCEL in data
				memset(buf, '0', sizeof(buf)); // reset buf

				strnbuf(4, buf, 0, "%04d", temp[2]); // PRESSURE in string format
				memcpy(&reply[14], buf, 4); //PRESSURE in data
				dbSet(logData, reply);
				break;
				}

		case COMPLETE:
				{
				printf("LOG COMPLETE!! \n");
				// FLUSH ALL DATA
				sysFlashErase(FlashPage(0x26000));
				sysFlashErase(FlashPage(0x26800));
				sysFlashErase(FlashPage(0x27600));
				sysFlashErase(FlashPage(0x28400));
				dataLog_counter = 0; // reset counter
				delChunk(kdataLogCounterType, NULL, delCompare);
				device_state = TRACKER_MODE;
				break;
				}

		default:
				{}
		}
	return 0;
	}
#else
#warning logRequest not currently supported!
STATUS logRequest(OBJECT o, bool isArray, byte index)
	{
	debug("%s\n", __FUNC__);
	return 0;
	}
#endif
